/*
 * Capacity.java
 *
 */

public class Capacity {

  private int sizeN;       // The size of the Hopfield model
  private int nStat;       // Number of sets of patterns to test for a given pattern set size
  private int pattInc;     // The pattern size increment
  private double maxFrac;  // The largest M/N fraction to test
  private double[] xPlot;  // Plot vector: X-axis M/N
  private double[] yPlot1; // Plot vector: Y1-axis P_err numerical estimation
  private double[] yPlot2; // Plot vector: Y2-axis P_err theory estimation 
	
  /**
   * The Capacity class. Constructor that takes a number of parameters.
   * 
   * @param sizeN The size of the Hopfield model (N)
   * @param nStat Number of sets of patterns to test for a given M
   * @param pattInc The pattern size increment
   * @param maxFrac The largest M/N fraction to test (M is the pattern set size)
   */
  public Capacity(int sizeN, int nStat, int pattInc, double maxFrac) {
    this.sizeN = sizeN;
    this.nStat = nStat;
    this.pattInc = pattInc;
    this.maxFrac = maxFrac; 
			
    /** 
     * Find the size of the plot arrays
     */
    int icnt = 0;
    for (int M = 2; M <= maxFrac*sizeN; M += pattInc) icnt++;
			
    xPlot = new double[icnt];
    yPlot1 = new double[icnt];
    yPlot2 = new double[icnt];
  }

	
  public static void main(String[] args) {

    /**
     * Some parameters
     */
    int hSize       = 100;  // Size of the Hopfield model
    int nStat       = 200;  // Number of sets of patterns to test for a given pattern set size
    int pattInc     = 5;    // The pattern size increment
    double maxFrac  = 0.5;  // The largest M/N fraction to test
      
    Capacity cap = new Capacity(hSize, nStat, pattInc, maxFrac);
    cap.EstimatepErr();
      
  }
	
  /**
   * This method will estimate P_err.
   */
  public void EstimatepErr() {
		
    /**
     * We need this class to do some plotting and mathematical
     */
    MyUtil util = new MyUtil();

    /**
     * Get the Hopfield network
     */
    Network network = new HebbianNetwork(sizeN);

    //--------------------------------------
    System.out.printf("NOT YET FINISHED!\n");
    System.exit(1);
    //---------------------------------------

    /**
     * Make a plot
     */
    util.plot(xPlot, yPlot1, yPlot2); 

  }
	
}
