/*
 * MyUtil.java
 *
 */

import javax.swing.JFrame;
import java.util.TreeSet;

public class MyUtil {

  /**
   * Chebyshev fitting formula for erf(z) from Numerical Recipes, 6.2
   * 
   * @param z Argument to the erf function.
   */
  public double erf(double z) {
    double t = 1.0 / (1.0 + 0.5 * Math.abs(z));

    // use Horner's method
    double ans = 1 - t * Math.exp( -z*z   -   1.26551223 +
				   t * ( 1.00002368 +
					 t * ( 0.37409196 + 
					       t * ( 0.09678418 + 
						     t * (-0.18628806 + 
							  t * ( 0.27886807 + 
								t * (-1.13520398 + 
								     t * ( 1.48851587 + 
									   t * (-0.82215223 + 
										t * ( 0.17087277))))))))));
    if (z >= 0) return  ans;
    else        return -ans;
  }
	
  /**
   * The shuffle method returns an array of shuffled integers from
   * 0 to n-1.
   * @param n The size of the array
   * @return An integer array of shuffled integers.
   */
  public int[] shuffle(int n) {
    int[] aux = new int[n];
    for (int i = 0; i < aux.length; i++) {
      aux[i] = i;
    }
    for (int i = 0; i < aux.length; i++) {
      int idx = (int)Math.floor(Math.random()*(aux.length-i));
      int tmp = aux[aux.length-i-1];
      aux[aux.length-i-1] = aux[idx];
      aux[idx] = tmp;
    }	
    return aux;
  }
	
	
  /**
   * A Small wrapper against Simon plot package
   * 
   * @param x x-values to be plotted 
   * @param y y-values to be plotted
   */
  public void plot(double[] x, double[] y) {
		
    JFrame frame = new JFrame();
    frame.setSize(600, 400);
    PlotPanel plot = new PlotPanel();
    frame.add(plot);

    Plotable xyPlot = new Plotable(x, y); 
    plot.addPlotable(xyPlot);
    frame.setVisible(true);		
  }

  /**
   * A Small wrapper against Simon plot package
   * 
   * @param x x-values to be plotted 
   * @param y first set of y-values to be plotted
   * @param y2 second set of y-values to be plotted
   */
  public void plot(double[] x, double[] y, double[] y2) {
		
    JFrame frame = new JFrame();
    frame.setSize(600, 400);
    PlotPanel plot = new PlotPanel();
    frame.add(plot);

    Plotable xyPlot = new Plotable(x, y); 
    Plotable xy2Plot = new Plotable(x, y2); 
    plot.addPlotable(xyPlot);
    plot.addPlotable(xy2Plot);
    frame.setVisible(true);		
  }


  public int[] flipBit(int[] patt, int nBits) {
    int[] newPatt = (int[])patt.clone();
		
    TreeSet<Integer> tmp = new TreeSet<Integer>();
    int nOk = 0;
    while( nOk < nBits ) {
      int idx = (int) (patt.length * Math.random());
      if( !tmp.contains(new Integer(idx) ) ) {
	newPatt[idx] *= -1;
	nOk += 1;
	tmp.add(new Integer(idx));
      }
    }
    return newPatt;
  }


  public int getDiff(int[] p1, int[] p2) {
    int diff = 0;
    for (int i = 0; i < p1.length; i++) {
      if( p1[i] != p2[i] ) diff += 1;
    }
    return diff;
  }

}
