/*
 * Plotable.java
 *
 * Created on March 3, 2008, 5:27 PM
 *
 */

import java.util.Vector;
import java.util.Map;

/**
 * This object contains x and y values for a data-set that can be plotted by 
 * PlotPanel. max/min of x and y can be accessed by the getXMax() etc. 
 * functions.
 * 
 * @author simon
 */

public class Plotable
{

  private double[] x,  y;
  private double xMax,  xMin,  yMax,  yMin;

  /** 
   * Creat an empty instance of Plotable 
   */
  public Plotable()
  {
  }

  /** 
   * Create an instance of Plotable from two arrays of values 
   */
  public Plotable(double[] x, double[] y)
  {
    setX(x);
    setY(y);
  }

  /** 
   * Create an instance of Plotable from two Vectors of values 
   */
  public Plotable(Vector<? extends Number> vx, Vector<? extends Number> vy)
  {
    setValues(vx, vy);
  }

  /** 
   * Create an instance of Plotable from a Map of pairs of values 
   */
  public Plotable(Map<? extends Number, ? extends Number> map)
  {
    setValues(map);
  }

  /** 
   * Set the x and y-values from two Vectors 
   */
  public void setValues(Vector<? extends Number> vx,
			Vector<? extends Number> vy)
  {
    setX(vx);
    setY(vy);
  }

  /** 
   * Set the x and y-values from a Map of pairs of values 
   */
  public void setValues(Map<? extends Number, ? extends Number> map)
  {
    setX(map.keySet());
    setY(map.values());

  }

  /** 
   * The x-values of the data-set 
   */
  public double[] getX()
  {
    return x;
  }

  /** 
   * The y-values of the data-set 
   */
  public double[] getY()
  {
    return y;
  }

  /** 
   * Set the x-values from an array 
   */
  public void setX(double[] x)
  {
    this.x = x;
  }

  /** 
   * Set the y-values from an array 
   */
  public void setY(double[] y)
  {
    this.y = y;
  }

  /** 
   * Set the x-vales from a Collection of Numbers 
   */
  public void setX(java.util.Collection<? extends Number> X)
  {
    x = new double[X.size()];
    int i = 0;
    for (Number xtmp : X) x[i++] = xtmp.doubleValue();
  }

  /** 
   * Set the y-values from a Collection of Numbers 
   */
  public void setY(java.util.Collection<? extends Number> Y)
  {
    y = new double[Y.size()];
    int i = 0;
    for (Number ytmp : Y) y[i++] = ytmp.doubleValue();
  }

  /**
   * @return max-value of x
   */
  public double getXMax()
  {
    xMax = x[0];
    for (double v : x) if (v > xMax) xMax = v;
    return xMax;
  }

  /**
   * @return min-value of x
   */
  public double getXMin()
  {
    xMin = x[0];
    for (double v : x) if (v < xMin) xMin = v;
    return xMin;
  }

  /**
   * @return max-value of y
   */
  public double getYMax()
  {
    yMax = y[0];
    for (double v : y) if (v > yMax) yMax = v;
    return yMax;
  }

  /**
   * @return min-value of y
   */
  public double getYMin()
  {
    yMin = y[0];
    for (double v : y) if (v < yMin) yMin = v;
    return yMin;
  }

}
