/*
 * PseudoInverseNetwork.java
 *
 */

import Jama.*;

public class PseudoInverseNetwork extends Network {

  public PseudoInverseNetwork(int sizeN) {
    super(sizeN);
  }

  /**
   * Train using the pseudo inverse approach.
   * 
   * @param data The patterns to for the training
   */
  @Override
    public void train(Data data) {
    // Compute the overlap matrix
    int nPatt = data.getPatterns().size();
    double[][] overlap = new double[nPatt][nPatt];
    for (int p1 = 0; p1 < nPatt; p1++) {
      for (int p2 = p1+1; p2 < nPatt; p2++) {
	overlap[p1][p2] = 0;
	for (int i = 0; i < neurons.length; i++) {
	  overlap[p1][p2] += data.getPattern(p1)[i] * data.getPattern(p2)[i];
	}
	overlap[p1][p2] /= neurons.length;
	overlap[p2][p1] = overlap[p1][p2];
      }
      overlap[p1][p1] = 1.0;
    }
    Matrix q = new Matrix(overlap);

    // Compute the inverse
    Matrix qInv = q.inverse();

    // Use the qInv matrix to compute new weights
    for (int i = 0; i < neurons.length; i++) {
      for (int j = i; j < neurons.length; j++) {
	weights[i][j] = 0;
	for (int k1 = 0; k1 < nPatt; k1++) {
	  double tmp = 0;
	  for (int k2 = 0; k2 < nPatt; k2++) {
	    tmp += qInv.get(k1,k2) * data.getPattern(k2)[j];
	  }
	  weights[i][j] += tmp * data.getPattern(k1)[i];
	}
	weights[i][j] /= neurons.length;
	weights[j][i] = weights[i][j];
      }
    }
  }
}
